from sqlalchemy import (
    Column,
    Integer,
    String,
    ForeignKey,
)

from ...aact_database import Base


class SponsorModel(Base):
    __tablename__ = 'sponsors'
    __table_args__ = (
        {'schema': 'ctgov'},
    )

    id = Column(Integer, primary_key=True)
    nct_id = Column(
        String,
        ForeignKey('ctgov.studies.nct_id'),
        nullable=False,
    )
    agency_class = Column(String)
    lead_or_collaborator = Column(String)
    name = Column(String)

    def __repr__(self):
        return "<Sponsor(id='{}', nct_id='{}', name='{}', agency_class={}, " \
               "lead_or_collaborator={})>".format(self.id, self.nct_id,
                                                  self.name,
                                                  self.agency_class,
                                                  self.lead_or_collaborator)



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)
from ...utils.utils import pre_load_date_fields


class NCTStudyResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    nct_id = fields.String(allow_none=False, validate=not_blank)
    brief_title = fields.String(allow_none=True)
    study_status = fields.String(allow_none=True)
    study_start_date = fields.DateTime(allow_none=True)
    last_update_submitted_qc_date = fields.DateTime(allow_none=True)
    phase = fields.String(allow_none=True)
    sponsors = fields.String(allow_none=True)
    industry_flag = fields.Boolean()
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = ['study_start_date', 'last_update_submitted_qc_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y-%m-%d',
        )
        return in_data



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError

from ...database import db_session
from ...resources.PTAB2Document.models import PTAB2DocumentModel
from .schemas import (
    PTAB2DocumentResourceSchema,
    PTAB2DocumentQueryParamsSchema,
    PTAB2DocumentPatchSchema,
)
from ...utils.utils import update_model


schema_resource = PTAB2DocumentResourceSchema()
schema_params = PTAB2DocumentQueryParamsSchema()
schema_patch = PTAB2DocumentPatchSchema()


class PTAB2Document(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (PTAB2DocumentResourceSchema)

        Returns:
            PTAB2DocumentResourceSchema

        Raises:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: PTAB2DocumentQueryParamsSchema

        Returns:
            List<PTAB2DocumentResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        ptab2_document_query = _build_query(params=data)
        response = schema_resource.dump(ptab2_document_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: PTAB2DocumentQueryParamsSchema

        Returns:
            PTAB2DocumentResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        ptab2_document_query = _build_query(params=data).one()
        response = schema_resource.dump(ptab2_document_query).data
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        ptab2_document_query = db_session.query(
            PTAB2DocumentModel).filter_by(id=id).one()
        try:
            db_session.delete(ptab2_document_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def update(id, params):
        """
        Args:
            id: int
            params: PTAB2DocumentPatchSchema

        Returns: PTAB2DocumentResourceSchema

        Raises:
            ValidationError
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        ptab2_document_query = db_session.query(PTAB2DocumentModel).filter_by(
            id=id).one()
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, ptab2_document_query)
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: PTAB2DocumentResourceSchema

        Returns:
            PTAB2DocumentResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        try:
            query_params = {
                'document_identifier': params['document_identifier'],
            }
            ptab2_document_query = _build_query(query_params).one()
            response = _helper_update(data, ptab2_document_query)
        except NoResultFound:
            response = _helper_create(data)
        return response

    @staticmethod
    def bulk_create(mappings):
        try:
            data, errors = schema_resource.load(mappings, many=True)
            db_session.bulk_insert_mappings(
                PTAB2DocumentModel,
                data,
            )
            db_session.commit()
        except SQLAlchemyError:
            db_session.rollback()
            raise


def _helper_create(data):
    new_ptab2_document = PTAB2DocumentModel(**data)
    try:
        db_session.add(new_ptab2_document)
        db_session.commit()
        ptab2_document_query = db_session.query(
            PTAB2DocumentModel).get(new_ptab2_document.id)
        response = schema_resource.dump(ptab2_document_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, ptab2_document_query):
    data['id'] = ptab2_document_query.id
    data['document_identifier'] = ptab2_document_query.document_identifier
    try:
        update_model(data, ptab2_document_query)
        db_session.commit()
        response = schema_resource.dump(ptab2_document_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        PTAB2DocumentModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('document_identifier'):
        q = q.filter_by(document_identifier=params.get('document_identifier'))
    if params.get('proceeding_number'):
        q = q.filter_by(proceeding_number=params.get('proceeding_number'))
    if 'has_smart_doc' in params:
        q = q.filter_by(has_smart_doc=params.get('has_smart_doc'))
    return q



import json
import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.PTAB2Decision import PTAB2Decision


class PTAB2DecisionResourceTestCase(TestCase):
    def setUp(self):
        super(PTAB2DecisionResourceTestCase, self).setUp()
        self.inst = PTAB2Decision()

        self.ptab2_decision1 = self.inst.create({
            'board_rulings': json.dumps([
                '35 USC 311',
                '35 USC 312',
                '35 USC 318',
                '37 CFR 42.73',
                '37 CFR 42.100'
            ]),
            'decision_date': '05-02-2019',
            'decision_type_category': 'Decision',
            'document_identifier': '169812221',
            'document_name': 'IPR2018-00107 Final Written Decision.pdf',
            'identifier': '1000',
            'issue_type': json.dumps(['102', '103']),
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Tedd Van Buskirk',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'St. Jude Medical, LLC',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_number': 'IPR2018-001',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '10135746',
            'respondent_counsel_name': 'Matthew Antonelli',
            'respondent_grant_date': '11-23-2004',
            'respondent_group_art_unit_number': '3738',
            'respondent_inventor_name': '-',
            'respondent_party_name': 'Snyders Heart Valve LLC',
            'respondent_patent_number': '6821297',
            'respondent_patent_owner_name': 'Snyders, Robert V.',
            'respondent_technology_center_number': '3700',
            'subdecision_type_category': 'Final Decision',
            'subproceeding_type_category': 'IPR',
        })

        self.ptab2_decision2 = self.inst.create({
            'board_rulings': json.dumps([
                '35 USC 311',
                '35 USC 312',
                '35 USC 318',
                '37 CFR 42.73',
                '37 CFR 42.100'
            ]),
            'decision_date': '05-02-2019',
            'decision_type_category': 'Decision',
            'document_identifier': '169812221',
            'document_name': 'IPR2018-00107 Final Written Decision.pdf',
            'identifier': '2000',
            'issue_type': json.dumps(['102', '103']),
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Tedd Van Buskirk',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'St. Jude Medical, LLC',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_number': 'IPR2018-002',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '10135746',
            'respondent_counsel_name': 'Matthew Antonelli',
            'respondent_grant_date': '11-23-2004',
            'respondent_group_art_unit_number': '3738',
            'respondent_inventor_name': '-',
            'respondent_party_name': 'Snyders Heart Valve LLC',
            'respondent_patent_number': '6821297',
            'respondent_patent_owner_name': 'Snyders, Robert V.',
            'respondent_technology_center_number': '3700',
            'subdecision_type_category': 'Final Decision',
            'subproceeding_type_category': 'IPR',
        })

        self.valid_data = {
            'board_rulings': json.dumps([
                '35 USC 311',
                '35 USC 312',
                '35 USC 318',
                '37 CFR 42.73',
                '37 CFR 42.100'
            ]),
            'decision_date': '05-02-2019',
            'decision_type_category': 'Decision',
            'document_identifier': '169812221',
            'document_name': 'IPR2018-00107 Final Written Decision.pdf',
            'identifier': '3000',
            'issue_type': json.dumps(['102', '103']),
            'petitioner_application_number_text': '-',
            'petitioner_counsel_name': 'Tedd Van Buskirk',
            'petitioner_grant_date': '-',
            'petitioner_group_art_unit_number': '-',
            'petitioner_inventor_name': '-',
            'petitioner_party_name': 'St. Jude Medical, LLC',
            'petitioner_patent_number': '-',
            'petitioner_patent_owner_name': '-',
            'petitioner_technology_center_number': '-',
            'proceeding_number': 'IPR2018-007',
            'proceeding_type_category': 'AIA Trial',
            'respondent_application_number_text': '10135746',
            'respondent_counsel_name': 'Matthew Antonelli',
            'respondent_grant_date': '11-23-2004',
            'respondent_group_art_unit_number': '3738',
            'respondent_inventor_name': '-',
            'respondent_party_name': 'Snyders Heart Valve LLC',
            'respondent_patent_number': '6821297',
            'respondent_patent_owner_name': 'Snyders, Robert V.',
            'respondent_technology_center_number': '3700',
            'subdecision_type_category': 'Final Decision',
            'subproceeding_type_category': 'IPR',
        }

    def test_create_violates_unique_constraint(self):
        data = copy.copy(self.valid_data)
        data['identifier'] = self.ptab2_decision1['identifier']
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
            **{
                'decision_date': '2019-05-02T00:00:00+00:00',
                'respondent_grant_date': '2004-11-23T00:00:00+00:00',
            }
        }
        second_equals_first(expected_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    @parameterized.expand([
        ('id', 'ptab2_decision1', 'id', 1),
        ('identifier', 'ptab2_decision1', 'identifier', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('identifier',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.ptab2_decision1[field_name],
        })
        second_equals_first(
            self.ptab2_decision1,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.ptab2_decision1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.ptab2_decision1['id']},
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'decision_date': self.valid_data['decision_date'],
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{'identifier': self.ptab2_decision1['identifier']},
        }
        resp = self.inst.upsert(data)
        expected_data = {
            **data,
            **{
                'decision_date': '2019-05-02T00:00:00+00:00',
                'respondent_grant_date': '2004-11-23T00:00:00+00:00',
            }
        }
        second_equals_first(
            expected_data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))



from marshmallow import ValidationError
from parameterized import parameterized

from test.backendtestcase import TestCase
from src.cs_models.resources.OrangeBookPatent import (
    OrangeBookPatent,
    OrangeBookPatentNotFoundException,
)


class OrangeBookPatentResourceTestCase(TestCase):
    def setUp(self):
        super(OrangeBookPatentResourceTestCase, self).setUp()
        self.inst = OrangeBookPatent()
        self.ob_patent1 = self.inst.create({
            'appl_no': '22101',
            'appl_type': 'N',
            'delist_flag': '',
            'drug_product_flag': '',
            'drug_substance_flag': '',
            'patent_expire_date': 'Oct 30, 2009',
            'patent_no': '6428810*PED',
            'patent_use_code': 'U-735',
            'product_no': '1',
            'submission_date': ''
        })

    def test_create_validation_error_missing_fields(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {},
        )

    def test_create_validation_error_blank_product_no(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {
                'appl_no': '123',
                'appl_type': 'N',
                'delist_flag': '',
                'drug_product_flag': '',
                'drug_substance_flag': '',
                'patent_expire_date': 'Oct 30, 2009',
                'patent_no': '6428810',
                'patent_use_code': 'U-905',
                'product_no': '',
                'submission_date': ''
            },
        )

    def test_create(self):
        data = {
            'appl_no': '2000',
            'appl_type': 'N',
            'delist_flag': '',
            'drug_product_flag': '',
            'drug_substance_flag': '',
            'patent_expire_date': 'Oct 30, 2009',
            'patent_no': '6428810*PED',
            'patent_use_code': 'No Use Code',
            'product_no': '12',
            'submission_date': ''
        }
        response = self.inst.create(data)
        self.assertEqual(data['product_no'], response['product_no'])
        self.assertEqual(data['patent_no'], response['patent_no'])
        self.assertEqual(data['patent_use_code'], response['patent_use_code'])

    @parameterized.expand([
        'appl_type',
        'appl_no',
        'product_no',
        'patent_no',
        'patent_use_code',
    ])
    def test_read_validation_error_blank(self, field_name):
        self.assertRaises(
            ValidationError,
            self.inst.read,
            {field_name: ''},
        )

    def test_read_all(self):
        response = self.inst.read({})
        self.assertEqual(len(response), 1)

    @parameterized.expand([
        ('appl_no', 1),
        ('product_no', 2),
        ('patent_no', 1),
    ])
    def test_read_w_params(self, field_name, expected_length):
        # setup
        new_ob_patent = self.inst.create({
            'appl_no': '22102',
            'appl_type': 'N',
            'delist_flag': '',
            'drug_product_flag': '',
            'drug_substance_flag': '',
            'patent_expire_date': 'Oct 30, 2009',
            'patent_no': '6428810',
            'patent_use_code': 'U-659',
            'product_no': '1',
            'submission_date': ''
        })

        response = self.inst.read({})
        self.assertEqual(len(response), 2)

        response = self.inst.read({field_name: new_ob_patent[field_name]})
        self.assertEqual(expected_length, len(response))

    def test_read_w_appl_nos(self):
        # setup
        new_ob_patent = self.inst.create({
            'appl_no': '22102',
            'appl_type': 'N',
            'delist_flag': '',
            'drug_product_flag': '',
            'drug_substance_flag': '',
            'patent_expire_date': 'Oct 30, 2009',
            'patent_no': '6428810',
            'patent_use_code': 'U-659',
            'product_no': '1',
            'submission_date': ''
        })

        response = self.inst.read({})
        self.assertEqual(len(response), 2)
        response = self.inst.read({'appl_nos': [new_ob_patent['appl_no'], 2]})
        self.assertEqual(1, len(response))
        self.assertIn(new_ob_patent, response)

        response = self.inst.read({'appl_nos': []})
        self.assertEqual(0, len(response))

    def test_update_not_found(self):
        invalid_id = 99999
        args = []
        kwargs = {'id': invalid_id, 'params': {
            'appl_no': '22101',
            'appl_type': 'N',
            'delist_flag': '',
            'drug_product_flag': '',
            'drug_substance_flag': '',
            'patent_expire_date': 'Oct 30, 2009',
            'patent_no': '6428810*PED',
            'patent_use_code': 'U-786',
            'product_no': '1',
            'submission_date': ''
        }}
        self.assertRaises(
            OrangeBookPatentNotFoundException,
            self.inst.update,
            *args,
            **kwargs
        )

    @parameterized.expand([
        'product_no',
        'patent_no',
        'patent_use_code',
    ])
    def test_update_validation_error_blank(self, field_name):
        args = []
        kwargs = {
            'id': self.ob_patent1['id'],
            'params': {
                'appl_no': '',
                'appl_type': 'N',
                'delist_flag': '',
                'drug_product_flag': '',
                'drug_substance_flag': '',
                'patent_expire_date': 'Oct 30, 2009',
                'patent_no': '6428810',
                'patent_use_code': '',
                'product_no': '123',
                'submission_date': ''
            },
        }
        kwargs['params'][field_name] = ''
        self.assertRaises(
            ValidationError,
            self.inst.update,
            *args,
            **kwargs
        )

    def test_update(self):
        new_data = {
            'appl_no': '22103',
            'appl_type': 'NN',
            'delist_flag': '',
            'drug_product_flag': '',
            'drug_substance_flag': '',
            'patent_expire_date': 'Oct 30, 2009',
            'patent_no': '6428810*PED',
            'patent_use_code': 'U-789',
            'product_no': '12',
            'submission_date': ''
        }
        response = self.inst.update(
            id=self.ob_patent1['id'],
            params=new_data,
        )
        self.assertEqual(response['id'], self.ob_patent1['id'])
        self.assertEqual(
            response['appl_no'],
            new_data['appl_no'],
        )
        self.assertEqual(
            response['product_no'],
            new_data['product_no'],
        )
        self.assertEqual(
            response['appl_type'],
            new_data['appl_type'],
        )

    def test_update_partial(self):
        new_data = {
            'appl_no': '123',
            'appl_type': 'NM',
            'patent_expire_date': '',
            'product_no': '0123123',
        }
        response = self.inst.update(
            id=self.ob_patent1['id'],
            params=new_data,
        )
        self.assertEqual(response['id'], self.ob_patent1['id'])
        self.assertEqual(
            response['appl_no'],
            new_data['appl_no'],
        )
        self.assertEqual(
            response['appl_type'],
            new_data['appl_type'],
        )
        self.assertEqual(
            response['product_no'],
            new_data['product_no'],
        )
        self.assertIsNone(response['patent_expire_date'])



from marshmallow import (
    Schema,
    fields,
    validate,
)


class ProductViewResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    intervention_name = fields.String(validate=not_blank, required=True)
    intervention_type = fields.String(allow_none=True)
    synonyms = fields.String()
    condition_id = fields.Integer(allow_none=True)
    disease_name = fields.String(allow_none=True)
    approved_condition = fields.Boolean()
    pharm_actions = fields.String()
    company_sec_id = fields.Integer(allow_none=True)
    company_ous_id = fields.Integer(allow_none=True)
    isonmarket = fields.Boolean(required=True)
    updated_at = fields.DateTime()



from sqlalchemy import (
    Column,
    Integer,
    Text,
    String,
    DateTime,
    ForeignKey,
    Boolean,
)
from datetime import datetime

from ...database import Base


class ProductViewModel(Base):
    __tablename__ = 'product_view'

    id = Column(Integer, primary_key=True)
    intervention_name = Column(String(128), nullable=False, index=True)
    intervention_type = Column(String(50), nullable=True)
    synonyms = Column(Text, nullable=True)
    condition_id = Column(
        Integer,
        ForeignKey('conditions.id'),
        nullable=True,
    )
    disease_name = Column(String(128), nullable=True)
    approved_condition = Column(Boolean, nullable=True)
    pharm_actions = Column(Text, nullable=True)
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    isonmarket = Column(Boolean, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



import json
import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.CrossRef import CrossRef


class CrossRefResourceTestCase(TestCase):
    def setUp(self):
        super(CrossRefResourceTestCase, self).setUp()
        self.inst = CrossRef()

        self.cross_ref1 = self.inst.create({
            'doi': '1000',
            'authors': json.dumps([
                {'family': 'Flechter', 'given': 'Shlomo'},
                {'family': 'Kott', 'given': 'Edna'},
            ]),
            'doi_url': 'http://dx.doi.org/10.1097/00002826-200201000-00002',
            'issn': json.dumps(['0362-5664']),
            'issue': '1',
            'journal': 'Clinical neuropharmacology',
            'pages': '11-5',
            'pmid': '11852290',
            'ref_type': 'journal-article',
            'text': 'Shlomo Flechter et al., Copolymer 1'
                    'Relapsing Forms of Multiple Sclerosis: Open '
                    'Multicenter Study of Alternate-Day Administration.',
            'title': 'Copolymer 1 (glatiramer acetate) in relapsing forms ',
            'volume': '25',
            'year': 2002,
        })

        self.cross_ref2 = self.inst.create({
            'doi': '2000',
            'authors': json.dumps([
                {'family': 'Flechter', 'given': 'Shlomo'},
            ]),
            'doi_url': 'http://dx.doi.org/10.1097/00002826-200201000-00002',
            'issn': json.dumps(['0362-5664']),
            'issue': '1123',
            'journal': 'Clinical neuropharmacology',
            'pages': '11-5',
            'pmid': '11852290',
            'ref_type': 'journal-article',
            'text': 'Shlomo Flechter et al., Copolymer 1'
                    'Relapsing Forms of Multiple Sclerosis: Open Multicenter '
                    'Study of Alternate-Day Administration.',
            'title': 'Copolymer 1 (glatiramer acetate) in relapsing forms ',
            'volume': '25',
            'year': 2002,
        })

        self.valid_data = {
            'doi': '3000',
            'authors': json.dumps([
                {'family': 'Flechter', 'given': 'Shlomo'},
            ]),
            'doi_url': 'http://dx.doi.org/10.1097/00002826-200201000-00002',
            'issn': json.dumps(['0362-5664']),
            'issue': '1123',
            'journal': 'Clinical neuropharmacology',
            'pages': '11-5',
            'pmid': '11852290',
            'ref_type': 'journal-article',
            'text': 'Shlomo Flechter et al., Copolymer 1'
            'Relapsing Forms of Multiple Sclerosis: Open Multicenter Study of '
            'Alternate-Day Administration.',
            'title': 'Copolymer 1 (glatiramer acetate) in relapsing forms ',
            'volume': '25',
            'year': 2002,
        }

    @parameterized.expand([
        ('doi',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create_violates_unique_constraint(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {'doi': self.cross_ref1['doi']},
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        second_equals_first(self.valid_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    @parameterized.expand([
        ('id', 'cross_ref1', 'id', 1),
        ('doi', 'cross_ref1', 'doi', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('doi',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.cross_ref1[field_name],
        })
        second_equals_first(
            self.cross_ref1,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.cross_ref1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.cross_ref1['id']},
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'doi_url': self.valid_data['doi_url'],
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{'doi': self.cross_ref1['doi']},
        }
        resp = self.inst.upsert(data)
        second_equals_first(
            data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Boolean,
)
from datetime import datetime

from ...database import Base


class PTAB2ProceedingModel(Base):
    __tablename__ = 'ptab2_proceedings'

    id = Column(Integer, primary_key=True)
    accorded_filing_date = Column(DateTime)
    max_document_date = Column(DateTime)
    decision_date = Column(DateTime)
    institution_decision_date = Column(DateTime)
    petitioner_application_number_text = Column(String(256))
    petitioner_counsel_name = Column(String(256))
    petitioner_grant_date = Column(DateTime)
    petitioner_group_art_unit_number = Column(String(20))
    petitioner_inventor_name = Column(String(100))
    petitioner_party_name = Column(String(256))
    petitioner_patent_number = Column(String(20))
    petitioner_patent_owner_name = Column(String(256))
    petitioner_technology_center_number = Column(String(256))
    proceeding_filing_date = Column(DateTime)
    proceeding_last_modified_date = Column(DateTime)
    proceeding_number = Column(String(20), nullable=False, unique=True, index=True)
    proceeding_status_category = Column(String(100))
    proceeding_type_category = Column(String(100))
    respondent_application_number_text = Column(String(100))
    respondent_counsel_name = Column(String(100))
    respondent_grant_date = Column(DateTime)
    respondent_group_art_unit_number = Column(String(20))
    respondent_inventor_name = Column(String(100))
    respondent_party_name = Column(String(256))
    respondent_patent_number = Column(String(20), index=True)
    respondent_patent_owner_name = Column(String(256))
    respondent_technology_center_number = Column(String(20))
    subproceeding_type_category = Column(String(20))
    proceeding_status = Column(String(100))
    appealed = Column(Boolean)
    respondent_subsidiary_id = Column(
        Integer,
        ForeignKey('subsidiaries.id'),
        nullable=True,
    )
    petitioner_subsidiary_id = Column(
        Integer,
        ForeignKey('subsidiaries.id'),
        nullable=True,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    Text,
    Boolean,
)

from ...database import Base


class NCTMetamapModel(Base):
    __tablename__ = "nct_metamap"

    id = Column(Integer, primary_key=True)
    nct_id = Column(String(128), nullable=False, index=True)
    cui = Column(String(50), nullable=False)
    preferred_name = Column(String(256), nullable=False)
    type = Column(String(128), nullable=False)
    score = Column(Integer, nullable=True)
    note = Column(Text, nullable=True)
    final_mapping = Column(Boolean, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class ICDResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    icd_code = fields.String(validate=not_blank, required=True)
    description = fields.String(validate=not_blank, required=True)
    class_kind = fields.String()
    is_leaf = fields.Boolean()
    updated_at = fields.DateTime()


class ICDQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    icd_code = fields.String(validate=not_blank)
    description = fields.String(validate=not_blank)


class ICDPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    icd_code = fields.String(validate=not_blank)
    description = fields.String(validate=not_blank)
    class_kind = fields.String()
    is_leaf = fields.Boolean()



from marshmallow import (
    Schema,
    fields,
    pre_load,
    ValidationError,
)


class RelatedMatterResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    ptab2_document_id = fields.Integer(required=True)
    related_pacer_case_id = fields.Integer()
    related_ptab2_proceeding_id = fields.Integer()
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def pre_load(self, in_data):
        if ('related_pacer_case_id' in in_data and
                'related_ptab2_proceeding_id' in in_data):
            raise ValidationError(
                'Cannot provide both `related_pacer_case_id` and '
                '`related_ptab2_proceeding_id` fields')

        if ('related_pacer_case_id' not in in_data and
                'related_ptab2_proceeding_id' not in in_data):
            raise ValidationError(
                'Should provide either `related_pacer_case_id` or '
                '`related_ptab2_proceeding_id` field')


class RelatedMatterQueryParamsSchema(Schema):
    id = fields.Integer()
    ptab2_document_id = fields.Integer()
    related_pacer_case_id = fields.Integer()
    related_ptab2_proceeding_id = fields.Integer()



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Boolean,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class ProceedingStageModel(Base):
    __tablename__ = 'proceeding_stages'

    id = Column(Integer, primary_key=True)
    ptab2_proceeding_id = Column(
        Integer,
        ForeignKey('ptab2_proceedings.id'),
        nullable=False,
    )
    stage = Column(String(128), nullable=False)
    is_active = Column(Boolean, nullable=False)
    filed_date = Column(DateTime)
    due_date = Column(DateTime)
    file_id = Column(Integer, ForeignKey("files.id"))
    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('ptab2_proceeding_id', 'stage'),)



from sqlalchemy import (
    Column,
    Integer,
    String,
)

from ...aact_database import Base


class MeshHeadingModel(Base):
    __tablename__ = 'mesh_headings'
    __table_args__ = (
        {'schema': 'ctgov'},
    )

    id = Column(Integer, primary_key=True)
    qualifier = Column(String)
    heading = Column(String)
    subcategory = Column(String)

    def __repr__(self):
        return "<MeshHeading(id='{}', qualifier='{}', " \
               "heading='{}', subcategory='{}'>"\
            .format(self.id, self.qualifier, self.heading,
                    self.subcategory)
